@@ -1,4 +1,48 @@
 
+/*
+ * The olsr.org Optimized Link-State Routing daemon version 2 (olsrd2)
+ * Copyright (c) 2004-2015, the olsr.org team - see HISTORY file
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * * Redistributions of source code must retain the above copyright
+ *   notice, this list of conditions and the following disclaimer.
+ * * Redistributions in binary form must reproduce the above copyright
+ *   notice, this list of conditions and the following disclaimer in
+ *   the documentation and/or other materials provided with the
+ *   distribution.
+ * * Neither the name of olsr.org, olsrd nor the names of its
+ *   contributors may be used to endorse or promote products derived
+ *   from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
+ * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
+ * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
+ * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
+ * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
+ * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
+ * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
+ * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
+ * POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Visit http://www.olsr.org for more information.
+ *
+ * If you find this software useful feel free to make a donation
+ * to the project. For more information see the website or contact
+ * the copyright holders.
+ *
+ */
+
+/**
+ * @file
+ */
+
 #include "common/common_types.h"
 #include "common/avl.h"
 #include "common/avl_comp.h"
@@ -11,25 +55,31 @@
 #include "dlep/dlep_writer.h"
 #include "dlep/dlep_session.h"
 
+/**
+ * internal constants of DLEP session
+ */
 enum {
+  /*! size increase step for DLEP value storage */
   SESSION_VALUE_STEP = 128,
 };
 
 static int _update_allowed_tlvs(struct dlep_session *session);
 static enum dlep_parser_error _parse_tlvstream(
     struct dlep_session *session, const uint8_t *buffer, size_t length);
-static enum dlep_parser_error _check_mandatory(
-    struct dlep_session *session, uint16_t signal_type);
-static enum dlep_parser_error _check_duplicate(
-    struct dlep_session *session, uint16_t signal_type);
-static enum dlep_parser_error _call_extension_processing(
-    struct dlep_session *parser, uint16_t signal_type);
+static enum dlep_parser_error _check_mandatory(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type);
+static enum dlep_parser_error _check_duplicate(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type);
+static enum dlep_parser_error _call_extension_processing(struct dlep_session *parser,
+    struct dlep_extension *ext, uint16_t signal_type);
 static struct dlep_parser_tlv *_add_session_tlv(
     struct dlep_session_parser *parser, uint16_t id);
+static enum dlep_parser_error _handle_extension(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type);
 static enum dlep_parser_error _process_tlvs(struct dlep_session *,
     uint16_t signal_type, uint16_t signal_length, const uint8_t *tlvs);
 static void _send_terminate(struct dlep_session *session);
-static void _cb_destination_timeout(void *);
+static void _cb_destination_timeout(struct oonf_timer_instance *);
 
 static struct oonf_class _tlv_class = {
     .name = "dlep reader tlv",
@@ -46,6 +96,9 @@ static struct oonf_timer_class _destination_ack_class = {
     .callback = _cb_destination_timeout,
 };
 
+/**
+ * Initialize DLEP session system
+ */
 void
 dlep_session_init(void) {
   oonf_class_add(&_tlv_class);
@@ -133,6 +186,10 @@ dlep_session_add(struct dlep_session *session, const char *l2_ifname,
   return 0;
 }
 
+/**
+ * Remove a DLEP session
+ * @param session dlep session
+ */
 void
 dlep_session_remove(struct dlep_session *session) {
   struct dlep_parser_tlv *tlv, *tlv_it;
@@ -179,6 +236,13 @@ dlep_session_terminate(struct dlep_session *session) {
   session->restrict_signal = DLEP_PEER_TERMINATION_ACK;
 }
 
+/**
+ * Update the list of active dlep extensions for a session
+ * @param session dlep session
+ * @param extvalues array with allowed DLEP sessions
+ * @param extcount number of bytes in array
+ * @return -1 if an error happened, 0 otherwise
+ */
 int
 dlep_session_update_extensions(struct dlep_session *session,
     const uint8_t *extvalues, size_t extcount) {
@@ -186,6 +250,8 @@ dlep_session_update_extensions(struct dlep_session *session,
   size_t count, i;
   uint16_t extid;
 
+  OONF_INFO(session->log_source, "Update session extension list");
+
   /* keep entry a few entries untouched, these are the base extensions */
   count = DLEP_EXTENSION_BASE_COUNT;
   for (i=0; i<extcount; i++) {
@@ -193,6 +259,7 @@ dlep_session_update_extensions(struct dlep_session *session,
 
     if (dlep_extension_get(ntohs(extid))) {
       count++;
+      OONF_INFO(session->log_source, "Add extension: %d", ntohs(extid));
     }
   }
 
@@ -219,6 +286,12 @@ dlep_session_update_extensions(struct dlep_session *session,
   return _update_allowed_tlvs(session);
 }
 
+/**
+ * Process data in DLEP session TCP input buffer
+ * @param tcp_session TCP session
+ * @param session DLEP session
+ * @return new TCP session state
+ */
 enum oonf_stream_session_state
 dlep_session_process_tcp(struct oonf_stream_session *tcp_session,
     struct dlep_session *session) {
@@ -255,6 +328,14 @@ dlep_session_process_tcp(struct oonf_stream_session *tcp_session,
   return STREAM_SESSION_ACTIVE;
 }
 
+/**
+ * Process the content of a buffer as DLEP signal(s)
+ * @param session dlep session
+ * @param buffer pointer to buffer
+ * @param length length of buffer
+ * @return number of bytes of buffer which were parsed and
+ *   can be removed, -1 if an error happened
+ */
 ssize_t
 dlep_session_process_buffer(
     struct dlep_session *session, const void *buffer, size_t length) {
@@ -284,6 +365,13 @@ dlep_session_process_buffer(
   return offset;
 }
 
+/**
+ * Process a DLEP signal/message
+ * @param session dlep session
+ * @param ptr pointer to buffer with DLEP signal/message
+ * @param length length of buffer
+ * @return numberr of bytes parsed, 0 if an error happened
+ */
 size_t
 dlep_session_process_signal(struct dlep_session *session,
     const void *ptr, size_t length) {
@@ -326,9 +414,9 @@ dlep_session_process_signal(struct dlep_session *session,
     return 0;
   }
 
-  OONF_DEBUG(session->log_source, "Process signal %u (length %u)"
-      " from %s (%" PRINTF_SIZE_T_SPECIFIER" bytes)",
-      signal_type, signal_length,
+  OONF_DEBUG_HEX(session->log_source, buffer, signal_length + 4,
+      "Process signal %u from %s (%" PRINTF_SIZE_T_SPECIFIER" bytes)",
+      signal_type,
       netaddr_socket_to_string(&nbuf, &session->remote_socket),
       length);
 
@@ -355,6 +443,12 @@ dlep_session_process_signal(struct dlep_session *session,
   return signal_length + 4;
 }
 
+/**
+ * Add a neighbor to the local DLEP storage
+ * @param session dlep session
+ * @param neigh neighbor MAC address
+ * @return pointer to dlep neighbor, NULL if out of memory
+ */
 struct dlep_local_neighbor *
 dlep_session_add_local_neighbor(struct dlep_session *session,
     const struct netaddr *neigh) {
@@ -375,7 +469,6 @@ dlep_session_add_local_neighbor(struct dlep_session *session,
 
   /* initialize timer */
   local->_ack_timeout.class = &_destination_ack_class;
-  local->_ack_timeout.cb_context = local;
 
   /* initialize backpointer */
   local->session = session;
@@ -383,6 +476,11 @@ dlep_session_add_local_neighbor(struct dlep_session *session,
   return local;
 }
 
+/**
+ * Remove a neighbor from the DLEP storage
+ * @param session dlep session
+ * @param local DLEP neighbor
+ */
 void
 dlep_session_remove_local_neighbor(struct dlep_session *session,
     struct dlep_local_neighbor *local) {
@@ -391,14 +489,12 @@ dlep_session_remove_local_neighbor(struct dlep_session *session,
   oonf_class_free(&_local_neighbor_class, local);
 }
 
-struct dlep_local_neighbor *
-dlep_session_get_local_neighbor(struct dlep_session *session,
-    const struct netaddr *neigh) {
-  struct dlep_local_neighbor *local;
-  return avl_find_element(&session->local_neighbor_tree,
-      neigh, local, _node);
-}
-
+/**
+ * Get the layer2 neigbor for a DLEP session MAC address
+ * @param session dlep session
+ * @param neigh MAC address of neighbor
+ * @return layer2 neighbor, NULL if not found
+ */
 struct oonf_layer2_neigh *
 dlep_session_get_local_l2_neighbor(struct dlep_session *session,
     const struct netaddr *neigh) {
@@ -406,31 +502,41 @@ dlep_session_get_local_l2_neighbor(struct dlep_session *session,
   struct oonf_layer2_neigh *l2neigh;
   struct oonf_layer2_net *l2net;
 #ifdef OONF_LOG_INFO
-  struct netaddr_str nbuf;
+  struct netaddr_str nbuf1, nbuf2;
 #endif
 
   dlep_neigh = dlep_session_get_local_neighbor(session, neigh);
   if (!dlep_neigh) {
     OONF_INFO(session->log_source, "Could not find local neighbor for %s",
-        netaddr_to_string(&nbuf, neigh));
+        netaddr_to_string(&nbuf1, neigh));
     return NULL;
   }
 
   l2net = oonf_layer2_net_get(session->l2_listener.name);
   if (!l2net) {
-    OONF_DEBUG(session->log_source, "Could not find l2net for new neighbor");
+    OONF_DEBUG(session->log_source, "Could not find l2net %s for new neighbor",
+        session->l2_listener.name);
     return NULL;
   }
 
   l2neigh = oonf_layer2_neigh_get(l2net, &dlep_neigh->neigh_addr);
   if (!l2neigh) {
     OONF_INFO(session->log_source, "Could not find l2neigh "
-        "for neighbor %s", netaddr_to_string(&nbuf, neigh));
+        "for neighbor %s (%s)", netaddr_to_string(&nbuf1, neigh),
+        netaddr_to_string(&nbuf2, &dlep_neigh->neigh_addr));
     return NULL;
   }
   return l2neigh;
 }
 
+/**
+ * Generate a DLEP signal/message
+ * @param session dlep session
+ * @param signal signal id
+ * @param neighbor neighbor MAC address the signal should refer to,
+ *   might be NULL
+ * @return -1 if an error happened, 0 otherwise
+ */
 static int
 _generate_signal(struct dlep_session *session, uint16_t signal,
     const struct netaddr *neighbor) {
@@ -485,6 +591,14 @@ _generate_signal(struct dlep_session *session, uint16_t signal,
   return 0;
 }
 
+/**
+ * Generate a DLEP signal/message
+ * @param session dlep session
+ * @param signal signal id
+ * @param neighbor neighbor MAC address the signal should refer to,
+ *   might be NULL
+ * @return -1 if an error happened, 0 otherwise
+ */
 int
 dlep_session_generate_signal(struct dlep_session *session, uint16_t signal,
     const struct netaddr *neighbor) {
@@ -495,6 +609,16 @@ dlep_session_generate_signal(struct dlep_session *session, uint16_t signal,
   return dlep_writer_finish_signal(&session->writer, session->log_source);
 }
 
+/**
+ * Generate a DLEP signal/message with a DLEP status TLV
+ * @param session dlep session
+ * @param signal signal id
+ * @param neighbor neighbor MAC address the signal should refer to,
+ *   might be NULL
+ * @param status DLEP status code
+ * @param msg ZERO terminated DLEP status text
+ * @return -1 if an error happened, 0 otherwise
+ */
 int
 dlep_session_generate_signal_status(struct dlep_session *session,
     uint16_t signal, const struct netaddr *neighbor,
@@ -510,6 +634,12 @@ dlep_session_generate_signal_status(struct dlep_session *session,
   return dlep_writer_finish_signal(&session->writer, session->log_source);
 }
 
+/**
+ * Get the value of the first DLEP TLV of a specific type
+ * @param session dlep session
+ * @param tlvtype DLEP TLV type
+ * @return DLEP value, NULL if not found
+ */
 struct dlep_parser_value *
 dlep_session_get_tlv_value(
     struct dlep_session *session, uint16_t tlvtype) {
@@ -533,6 +663,12 @@ dlep_session_get_tlv_value(
   return value;
 }
 
+/**
+ * Update the list of allowed TLVs based on the list of allowed
+ * extensions
+ * @param session dlep session
+ * @return -1 if extensions are inconsistent, 0 otherwise
+ */
 static int
 _update_allowed_tlvs(struct dlep_session *session) {
   struct dlep_session_parser *parser;
@@ -588,29 +724,44 @@ _update_allowed_tlvs(struct dlep_session *session) {
   return 0;
 }
 
+/**
+ * Check constraints of extensions and call the relevant callbacks
+ * @param session dlep session
+ * @param ext dlep extension
+ * @param signal_type signal type
+ * @return dlep parser error, 0 of everything is fine
+ */
 static enum dlep_parser_error
-_process_tlvs(struct dlep_session *session,
-    uint16_t signal_type, uint16_t signal_length, const uint8_t *tlvs) {
+_handle_extension(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type) {
   enum dlep_parser_error result;
+  bool active;
+  size_t e;
 
-  OONF_DEBUG(session->log_source, "Parse signal %u with length %u",
-      signal_type, signal_length);
+  active = false;
 
-  /* start at the beginning of the tlvs */
-  if ((result = _parse_tlvstream(session, tlvs, signal_length))) {
-    OONF_DEBUG(session->log_source, "parse_tlvstream result: %d", result);
-    return result;
+  /* only handle active extensions */
+  for (e=0; e<session->parser.extension_count; e++) {
+    if (session->parser.extensions[e] == ext) {
+      active = true;
+      break;
+    }
   }
-  if ((result = _check_mandatory(session, signal_type))) {
+  if (!active) {
+    /* not active at the moment */
+    return DLEP_NEW_PARSER_OKAY;
+  }
+
+  if ((result = _check_mandatory(session, ext, signal_type))) {
     OONF_DEBUG(session->log_source, "check_mandatory result: %d", result);
     return result;
   }
-  if ((result = _check_duplicate(session, signal_type))) {
+  if ((result = _check_duplicate(session, ext, signal_type))) {
     OONF_DEBUG(session->log_source, "check_duplicate result: %d", result);
     return result;
   }
 
-  if ((result = _call_extension_processing(session, signal_type))) {
+  if ((result = _call_extension_processing(session, ext, signal_type))) {
     OONF_DEBUG(session->log_source,
         "extension processing failed: %d", result);
     return result;
@@ -619,6 +770,39 @@ _process_tlvs(struct dlep_session *session,
   return DLEP_NEW_PARSER_OKAY;
 }
 
+/**
+ * Parse a DLEP tlv
+ * @param session dlep session
+ * @param signal_type dlep signal/message type
+ * @param signal_length signal/message length
+ * @param tlvs pointer to bytearray with TLVs
+ * @return dlep parser status
+ */
+static enum dlep_parser_error
+_process_tlvs(struct dlep_session *session,
+    uint16_t signal_type, uint16_t signal_length, const uint8_t *tlvs) {
+  enum dlep_parser_error result;
+  struct dlep_extension *ext;
+
+  /* start at the beginning of the tlvs */
+  if ((result = _parse_tlvstream(session, tlvs, signal_length))) {
+    OONF_DEBUG(session->log_source, "parse_tlvstream result: %d", result);
+    return result;
+  }
+
+  avl_for_each_element(dlep_extension_get_tree(), ext, _node) {
+    if ((result = _handle_extension(session, ext, signal_type))) {
+      return result;
+    }
+  }
+
+  return DLEP_NEW_PARSER_OKAY;
+}
+
+/**
+ * terminate a DLEP session
+ * @param session dlep session
+ */
 static void
 _send_terminate(struct dlep_session *session) {
   if (session->restrict_signal != DLEP_PEER_DISCOVERY
@@ -630,16 +814,27 @@ _send_terminate(struct dlep_session *session) {
   }
 }
 
+/**
+ * Callback when a destination up/down signal times out
+ * @param ptr timer instance that fired
+ */
 static void
-_cb_destination_timeout(void *ptr) {
+_cb_destination_timeout(struct oonf_timer_instance *ptr) {
   struct dlep_local_neighbor *local;
 
-  local = ptr;
+  local = container_of(ptr, struct dlep_local_neighbor, _ack_timeout);
   if (local->session->cb_destination_timeout) {
     local->session->cb_destination_timeout(local->session, local);
   }
 }
 
+/**
+ * parse a stream of DLEP tlvs
+ * @param session dlep session
+ * @param buffer TLV buffer
+ * @param length buffer size
+ * @return DLEP parser status
+ */
 static enum dlep_parser_error
 _parse_tlvstream(struct dlep_session *session,
     const uint8_t *buffer, size_t length) {
@@ -736,86 +931,96 @@ _parse_tlvstream(struct dlep_session *session,
   return DLEP_NEW_PARSER_OKAY;
 }
 
+/**
+ * Check if all mandatory TLVs were found
+ * @param session dlep session
+ * @param ext dlep extension
+ * @param signal_type dlep signal/message type
+ * @return dlep parser status
+ */
 static enum dlep_parser_error
-_check_mandatory(struct dlep_session *session, uint16_t signal_type) {
+_check_mandatory(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type) {
   struct dlep_session_parser *parser;
   struct dlep_parser_tlv *tlv;
   struct dlep_extension_signal *extsig;
-  struct dlep_extension *ext;
-  size_t e,s,t;
+  size_t s,t;
 
   parser = &session->parser;
 
-  for (e = 0; e < parser->extension_count; e++) {
-    ext = parser->extensions[e];
+  extsig = NULL;
+  for (s = 0; s < ext->signal_count; s++) {
+    if (ext->signals[s].id == signal_type) {
+      extsig = &ext->signals[s];
+      break;
+    }
+  }
 
-    extsig = NULL;
-    for (s = 0; s < ext->signal_count; s++) {
-      if (ext->signals[s].id == signal_type) {
-        extsig = &ext->signals[s];
-        break;
-      }
+  if (!extsig) {
+    return DLEP_NEW_PARSER_OKAY;
+  }
+
+  for (t = 0; t < extsig->mandatory_tlv_count; t++) {
+    tlv = dlep_parser_get_tlv(parser, extsig->mandatory_tlvs[t]);
+    if (!tlv) {
+      OONF_WARN(session->log_source, "Could not find tlv data for"
+          " mandatory TLV %u in extension %d",
+          extsig->mandatory_tlvs[t], ext->id);
+      return DLEP_NEW_PARSER_INTERNAL_ERROR;
     }
 
-    if (extsig) {
-      for (t = 0; t < extsig->mandatory_tlv_count; t++) {
-        tlv = dlep_parser_get_tlv(parser, extsig->mandatory_tlvs[t]);
-        if (!tlv) {
-          OONF_WARN(session->log_source, "Could not find tlv data for"
-              " mandatory TLV %u in extension %d",
-              extsig->mandatory_tlvs[t], ext->id);
-          return DLEP_NEW_PARSER_INTERNAL_ERROR;
-        }
-
-        if (tlv->tlv_first == -1) {
-          OONF_WARN(session->log_source, "Missing mandatory TLV"
-              " %u in extension %d",
-              extsig->mandatory_tlvs[t], ext->id);
-          return DLEP_NEW_PARSER_MISSING_MANDATORY_TLV;
-        }
-      }
+    if (tlv->tlv_first == -1) {
+      OONF_WARN(session->log_source, "Missing mandatory TLV"
+          " %u in extension %d",
+          extsig->mandatory_tlvs[t], ext->id);
+      return DLEP_NEW_PARSER_MISSING_MANDATORY_TLV;
     }
   }
   return DLEP_NEW_PARSER_OKAY;
 }
 
+/**
+ * Check if all duplicate TLVs were allowed to be duplicates
+ * @param session dlep session
+ * @param ext dlep extension
+ * @param signal_type dlep signal/message type
+ * @return dlep parser status
+ */
 static enum dlep_parser_error
-_check_duplicate(struct dlep_session *session, uint16_t signal_type) {
+_check_duplicate(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type) {
   struct dlep_session_parser *parser;
   struct dlep_parser_tlv *tlv;
   struct dlep_extension_signal *extsig;
-  struct dlep_extension *ext = NULL;
-  size_t e, s, t;
+  size_t s, t, dt;
   bool okay;
 
   parser = &session->parser;
 
-  avl_for_each_element(&parser->allowed_tlvs, tlv, _node) {
-    if (tlv->tlv_first == tlv->tlv_last) {
+  extsig = NULL;
+  for (s = 0; s < ext->signal_count; s++) {
+    extsig = &ext->signals[s];
+    if (ext->signals[s].id == signal_type) {
+      extsig = &ext->signals[s];
+      break;
+    }
+  }
+
+  if (!extsig) {
+    return DLEP_NEW_PARSER_OKAY;
+  }
+
+  for (t = 0; t < extsig->supported_tlv_count; t++) {
+    tlv = avl_find_element(&parser->allowed_tlvs, &extsig->supported_tlvs[t], tlv, _node);
+    if (tlv == NULL || tlv->tlv_first == tlv->tlv_last) {
       continue;
     }
 
-    /* multiple tlvs of the same kind */
     okay = false;
-    for (e = 0; e < parser->extension_count && !okay; e++) {
-      ext = parser->extensions[e];
-
-      extsig = NULL;
-      for (s = 0; s < ext->signal_count; s++) {
-        extsig = &ext->signals[s];
-        if (ext->signals[s].id == signal_type) {
-          extsig = &ext->signals[s];
-          break;
-        }
-      }
-
-      if (extsig) {
-        for (t = 0; t < extsig->duplicate_tlv_count; t++) {
-          if (extsig->duplicate_tlvs[t] == tlv->id) {
-            okay = true;
-            break;
-          }
-        }
+    for (dt = 0; dt < extsig->duplicate_tlv_count; dt++) {
+      if (extsig->duplicate_tlvs[dt] == tlv->id) {
+        okay = true;
+        break;
       }
     }
     if (!okay) {
@@ -827,39 +1032,48 @@ _check_duplicate(struct dlep_session *session, uint16_t signal_type) {
   return DLEP_NEW_PARSER_OKAY;
 }
 
+/**
+ * Call extension processing hooks for parsed signal/message
+ * @param session dlep session
+ * @param ext dlep_extension
+ * @param signal_type dlep signal/message type
+ * @return dlep parser status
+ */
 static enum dlep_parser_error
-_call_extension_processing(struct dlep_session *session, uint16_t signal_type) {
-  struct dlep_extension *ext;
-  size_t e, s;
+_call_extension_processing(struct dlep_session *session,
+    struct dlep_extension *ext, uint16_t signal_type) {
+  size_t s;
 
-  for (e=0; e<session->parser.extension_count; e++) {
-    ext = session->parser.extensions[e];
-
-    for (s=0; s<ext->signal_count; s++) {
-      if (ext->signals[s].id != signal_type) {
-        continue;
-      }
-
-      if (session->radio) {
-        if (ext->signals[s].process_radio(ext, session)) {
-          OONF_DEBUG(session->log_source,
-              "Error in radio signal processing of extension '%s'", ext->name);
-          return -1;
-        }
-      }
-      else {
-        if (ext->signals[s].process_router(ext, session)) {
-          OONF_DEBUG(session->log_source,
-              "Error in router signal processing of extension '%s'", ext->name);
-          return -1;
-        }
-      }
-      break;
+  for (s=0; s<ext->signal_count; s++) {
+    if (ext->signals[s].id != signal_type) {
+      continue;
     }
+
+    if (session->radio) {
+      if (ext->signals[s].process_radio(ext, session)) {
+        OONF_DEBUG(session->log_source,
+            "Error in radio signal processing of extension '%s'", ext->name);
+        return -1;
+      }
+    }
+    else {
+      if (ext->signals[s].process_router(ext, session)) {
+        OONF_DEBUG(session->log_source,
+            "Error in router signal processing of extension '%s'", ext->name);
+        return -1;
+      }
+    }
+    break;
   }
   return DLEP_NEW_PARSER_OKAY;
 }
 
+/**
+ * Add a TLV to the allowed TLVs of a DLEP session
+ * @param parser dlep session parser
+ * @param id DLEP TLV id
+ * @return dlep parser TLV, NULL if out of memory
+ */
 static struct dlep_parser_tlv *
 _add_session_tlv(struct dlep_session_parser *parser, uint16_t id) {
   struct dlep_parser_tlv *tlv;
